from django.utils.functional import cached_property
from django.views.generic import ListView

from servala.core.models import Service
from servala.frontend.forms.service import ServiceFilterForm
from servala.frontend.views.mixins import OrganizationViewMixin


class OrganizationServicesView(OrganizationViewMixin, ListView):
    """View to display all available services for an organization."""

    template_name = "frontend/organizations/services.html"
    context_object_name = "services"
    model = Service
    permission_type = "view"

    def get_queryset(self):
        """Return all services."""
        services = Service.objects.all().select_related("category")
        if self.filter_form.is_valid():
            services = self.filter_form.filter_queryset(services)
        return services

    @cached_property
    def filter_form(self):
        return ServiceFilterForm(data=self.request.GET or None)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["filter_form"] = self.filter_form
        return context
